#include "Cube.hpp"


// Static geometry for the cube

static const GLfloat _vertices[] = {
    -1, -1, 1,
    1, -1, 1,
    -1, 1, 1,
    1, 1, 1,
    -1, -1, -1,
    1, -1, -1,
    -1, 1, -1,
    1, 1, -1
};

static const GLubyte _colors[] = {
    0, 0, 255,
    0, 255, 0,
    255, 0, 0,
    0, 255, 255,
    255, 0, 255,
    255, 255, 0,
    255, 255, 255,
    0, 0, 0
};

enum cube_vertex_meanings {
    TOP_UL,
    TOP_UR,
    TOP_LL,
    TOP_LR,
    BOT_UL,
    BOT_UR,
    BOT_LL,
    BOT_LR,
};

static const GLubyte _indices[] = {
    // Top face
    TOP_UR, TOP_UL, TOP_LL, TOP_LR,

    // Left face
    TOP_LL, TOP_UL, BOT_UL, BOT_LL,

    // Right face
    TOP_UR, TOP_LR, BOT_LR, BOT_UR,

    // Up face
    TOP_UL, TOP_UR, BOT_UR, BOT_UL,

    // Down face
    TOP_LR, TOP_LL, BOT_LL, BOT_LR,

    // Bottom face
    BOT_UL, BOT_UR, BOT_LR, BOT_LL
};


// Ctor
Cube::Cube()
{
    _valid = false;
}

// Dtor
Cube::~Cube()
{
    clean();
}

// Initializing the vertex buffer objects
void Cube::init()
{
    if (GLEE_VERSION_1_5)
    {
        // Generate two vertex buffer objects
        glGenBuffers(3, _vbo);

        // Bind buffer 0 to contain the color/vertex data
        glBindBuffer(GL_ARRAY_BUFFER, _vbo[0]);

        // Load the vertex data into buffer 0
        glBufferData(GL_ARRAY_BUFFER, sizeof(_vertices), _vertices, GL_STATIC_DRAW);

        // Bind buffer 0 to contain the color/vertex data
        glBindBuffer(GL_ARRAY_BUFFER, _vbo[1]);

        // Load the vertex data into buffer 0
        glBufferData(GL_ARRAY_BUFFER, sizeof(_colors), _colors, GL_STATIC_DRAW);

        // Bind buffer 1 to contain the vertex array data
        glBindBuffer(GL_ELEMENT_ARRAY_BUFFER, _vbo[2]);

        // Load the index data into buffer 1
        glBufferData(GL_ELEMENT_ARRAY_BUFFER, sizeof(_indices), _indices, GL_STATIC_DRAW);

        OGLDIF_CHECK_ERROR;
    }

    _valid = true;
}

// Draw the cube
void Cube::draw()
{
    if (!_valid)
        init();

    // Push client vertex array state
    glPushClientAttrib(GL_CLIENT_VERTEX_ARRAY_BIT);

    // Enable using vertex arrays
    glEnableClientState(GL_VERTEX_ARRAY);
    glEnableClientState(GL_COLOR_ARRAY);

    // If OGL version 1.5 or newer,
    if (GLEE_VERSION_1_5)
    {
        // Bind buffer 0 for the vertex array
        glBindBuffer(GL_ARRAY_BUFFER, _vbo[0]);

        // Specify the vertex data from buffer 0
        glVertexPointer(3, GL_FLOAT, 0, bufferObjectPtr(0));

        // Bind buffer 1 for the color array
        glBindBuffer(GL_ARRAY_BUFFER, _vbo[1]);

        // Specify the color data from buffer 0
        glColorPointer(3, GL_UNSIGNED_BYTE, 0, bufferObjectPtr(0));

        // Bind buffer 1 for the index array
        glBindBuffer(GL_ELEMENT_ARRAY_BUFFER, _vbo[2]);

        // Use glDrawRangeElements() for best performance
        glDrawRangeElements(GL_QUADS, 0, 4*6-1, 4*6, GL_UNSIGNED_BYTE, bufferObjectPtr(0));
    }
    else
    {
        // Set the vertex pointer directly
        glVertexPointer(3, GL_FLOAT, 0, _vertices);

        // Set the color pointer directly
        glColorPointer(3, GL_UNSIGNED_BYTE, 0, _colors);

        // If OGL version 1.2 or newer,
        if (GLEE_VERSION_1_2)
        {
            // Use glDrawRangeElements() for best performance
            glDrawRangeElements(GL_QUADS, 0, 4*6-1, 4*6, GL_UNSIGNED_BYTE, _indices);
        }
        else
        {
            // Use glDrawElements() as a fallback
            glDrawElements(GL_QUADS, 4*6, GL_UNSIGNED_BYTE, _indices);
        }
    }

    // Pop client vertex array state
    glPopClientAttrib();

    OGLDIF_CHECK_ERROR;
}

// Clean up
void Cube::clean()
{
    if (_valid)
    {
        _valid = false;

        if (GLEE_VERSION_1_5)
        {
            glDeleteBuffers(3, _vbo);
        }
    }
}
